/*
 * precompile.h
 *
 * OPAL application source file for EXTREMELY simple Multipoint Conferencing Unit
 *
 * Copyright (c) 2008 Vox Lucida Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Open Phone Abstraction Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Revision: 24766 $
 * $Author: rjongbloed $
 * $Date: 2010-09-30 02:18:52 -0500 (Thu, 30 Sep 2010) $
 */

#include <ptlib.h>
#include <ptclib/cli.h>
#include <opal/console_mgr.h>
#include <opal/opalmixer.h>
#include <opal/ivr.h>
#include <opal/pcss.h>
#include <sip/sipep.h>
#include <h323/h323ep.h>


// End of File ///////////////////////////////////////////////////////////////
