/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 1.3.40
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.opalvoip;

public class OpalProductDescription {
  private long swigCPtr;
  protected boolean swigCMemOwn;

  protected OpalProductDescription(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(OpalProductDescription obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        OPALJNI.delete_OpalProductDescription(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public void setM_vendor(String value) {
    OPALJNI.OpalProductDescription_m_vendor_set(swigCPtr, this, value);
  }

  public String getM_vendor() {
    return OPALJNI.OpalProductDescription_m_vendor_get(swigCPtr, this);
  }

  public void setM_name(String value) {
    OPALJNI.OpalProductDescription_m_name_set(swigCPtr, this, value);
  }

  public String getM_name() {
    return OPALJNI.OpalProductDescription_m_name_get(swigCPtr, this);
  }

  public void setM_version(String value) {
    OPALJNI.OpalProductDescription_m_version_set(swigCPtr, this, value);
  }

  public String getM_version() {
    return OPALJNI.OpalProductDescription_m_version_get(swigCPtr, this);
  }

  public void setM_t35CountryCode(long value) {
    OPALJNI.OpalProductDescription_m_t35CountryCode_set(swigCPtr, this, value);
  }

  public long getM_t35CountryCode() {
    return OPALJNI.OpalProductDescription_m_t35CountryCode_get(swigCPtr, this);
  }

  public void setM_t35Extension(long value) {
    OPALJNI.OpalProductDescription_m_t35Extension_set(swigCPtr, this, value);
  }

  public long getM_t35Extension() {
    return OPALJNI.OpalProductDescription_m_t35Extension_get(swigCPtr, this);
  }

  public void setM_manufacturerCode(long value) {
    OPALJNI.OpalProductDescription_m_manufacturerCode_set(swigCPtr, this, value);
  }

  public long getM_manufacturerCode() {
    return OPALJNI.OpalProductDescription_m_manufacturerCode_get(swigCPtr, this);
  }

  public OpalProductDescription() {
    this(OPALJNI.new_OpalProductDescription(), true);
  }

}
